package org.netkernel.module.standard;

import java.net.URI;
import java.util.List;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.IModuleFactory;
import org.netkernel.layer0.util.SpaceClassLoader;
import org.netkernel.layer0.util.XMLReadable;

public class StandardModuleFactory implements IModuleFactory
{

	public IModule buildModule(URI aModuleURI, IKernel aKernel)
			throws Exception
	{
		boolean suitable;
		XMLReadable defn=null;
		StringBuilder errors=new StringBuilder();
		errors.append(aModuleURI.toString()).append(" : ");
		try
		{	
			defn=BootUtils.getXMLFrom(aModuleURI,StandardModule.MODULE_DEFN);
			suitable = defn.getText("/module/@version").equals("2.0");
			if(!suitable)
			{	errors.append("Version must be 2.0 ");
			}
			//Validate Java Version
			String jvm=null;
			jvm=defn.getText("/module/system/jvm");
			if(!"".equals(jvm))
			{	Double version=Double.parseDouble(jvm);
				boolean validjvm=JAVA_VERSION>=version;
				if(!validjvm)
				{	errors.append("Java version must be ").append(jvm).append(" or higher ");					
				}
				suitable = suitable && validjvm;
			}
		}
		catch (Exception e)
		{	suitable=false;
		}
		finally { }
		if (suitable)
		{	ClassLoader parent = this.getClass().getClassLoader();
			List exports = defn.getTexts("/module/system/classloader/exports/match",true);
			if (exports.size()==0) exports=null;
			SpaceClassLoader cl=BootUtils.getClassLoaderFrom(aModuleURI,parent,"lib",exports,aModuleURI.toString(),aKernel);
			return new StandardModule(aModuleURI,defn,aKernel, cl);
		}
		else
		{	aKernel.getLogger().log(aKernel.getLogger().SEVERE, this, errors.toString());
			return null;
		}
	}

	public static int JAVA_VERSION = getMajorJavaVersion ();

	public static int getMajorJavaVersion()
	{	int majorVersion;
		String javaVersion=System.getProperty("java.version");
		String[] versionParts=javaVersion.split("\\.");
		if (versionParts.length>0)
		{	int part0=Integer.parseInt(versionParts[0]);
			if (part0==1 && versionParts.length>1)
			{	majorVersion=Integer.parseInt(versionParts[1]);
			}
			else
			{	majorVersion=part0;
			}
		}
		else
		{	majorVersion=-1;
		}
		return majorVersion;
	}
}
