package org.netkernel.module.standard.builtin.mapper;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.impl.MetadataUtils;
import org.netkernel.layer0.meta.impl.SpaceElementsImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.TupleList;
import org.netkernel.module.standard.StandardSpace;
import org.netkernel.module.standard.builtin.mapper.MapperConfig.Mapping;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.module.standard.endpoint.StandardEndpointImpl;
import org.netkernel.urii.ClassLoaderWithExports;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.ISpace;

public class MapperPrivateSpace extends StandardSpace
{
	public MapperPrivateSpace(Mapping[] aMappings, MapperOverlay aMapper) throws NKFException
	{	//super(aOwner,createEndpoints(aMappings),"MapperPrivateSpace",null);
		super(aMapper.getOwningSpace(),aMapper.getOwningSpace().getOwningModule(),null);
		setMappings(createEndpoints(aMappings));
	}
	
	@Override
	public ClassLoaderWithExports getClassLoader()
	{	return null;
	}
	
	private static IStandardEndpoint[] createEndpoints(Mapping[] aMappings)
	{	IStandardEndpoint ep = new MapperPrivateEndpoint(aMappings);
		return new IStandardEndpoint[]{ep};
	}
	
	private static class MapperPrivateEndpoint extends StandardEndpointImpl
	{
		private Mapping[] mMappings;
		private ISpaceElements mElements;
		private Map<String, Mapping> mMappingsById;
		
		public MapperPrivateEndpoint(Mapping[] aMappings)
		{	declareThreadSafe();
			mMappings=aMappings;
			
			ISpaceElements result;
			mMappingsById = new HashMap<String, Mapping>();
			TupleList parts = new TupleList(3,mMappings.length);
			for (int i=0; i<mMappings.length; i++)
			{	Mapping m=mMappings[i];
				String id = m.getId();
				mMappingsById.put(id, m);
				
				ISpace delegateSpace=null;
				
				parts.put(new SimpleIdentifierImpl(id), delegateSpace,this);
			}
			mElements = new SpaceElementsImpl(parts);			
		}
		
		public ISpaceElements getParts(INKFRequestContext aContext) throws Exception
		{	return mElements;
		}
		
		public void onResolve(INKFResolutionContext aContext)
		{	INKFRequestReadOnly toResolve = aContext.getRequestToResolve();
			for (Mapping mapping : mMappings)
			{	IEndpointMeta meta=mapping.getMeta();
				if (MetadataUtils.match((IEndpointMeta)meta, toResolve, aContext,mapping.getId()))
				{	String endpointId = mapping.getId();
					aContext.createResolutionResponse(this, endpointId);
					break;
				}
			}
		}
		
		public void onRequest(INKFRequestContext aContext) throws Exception
		{
			INKFRequestReadOnly req = aContext.getThisRequest();
			if (req.getVerb()==INKFRequestReadOnly.VERB_META)
			{	String identifier = aContext.getThisRequest().getIdentifier();
				Mapping mapping = mMappingsById.get(identifier);
				aContext.createResponseFrom(mapping.getMeta());
			}
			else
			{	String identifier=aContext.getThisRequest().getResolvedElementId();
				Mapping mapping = mMappingsById.get(identifier);
				mapping.onRequest(aContext);
			}
		}
	}
}
